package com.nifty.homepage2.jyube.hanabi;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.media.AudioManager;
//import android.media.MediaPlayer;
import android.media.SoundPool;
import android.support.v4.view.MotionEventCompat;
import android.util.Log;
import android.view.MotionEvent;
import android.view.SurfaceView;

public class MainView extends SurfaceView implements Runnable {
	public static final int SOUND_MAX = 6;
	public static final int SOUND_ID_DONE = 0;

	private boolean terminated = false;
	private boolean paused = false;
	private List<FireBall> balls;
	//private MediaPlayer player;
	private SoundPool sound;
	private List<Integer> sounds;
	private Map<Integer, Screen.Position> touchs;
	private List<Hanabi> hanabis;

	@SuppressLint("UseSparseArrays")
	public MainView(Context context) {
		super(context);
		balls = new ArrayList<FireBall>();
		touchs = Collections.synchronizedMap(
				new HashMap<Integer, Screen.Position>());
		hanabis = new ArrayList<Hanabi>();
/*
		player = MediaPlayer.create(
				context, R.raw.whiteberry_natumaturi);
		player.setLooping(true);
*/
		sound = new SoundPool(
				SOUND_MAX, AudioManager.STREAM_MUSIC, 0);
		sounds = new ArrayList<Integer>();
		sounds.add(sound.load(context, R.raw.done01, 1));
/*
		sounds.add(sound.load(context, R.raw.done02, 1));
		sounds.add(sound.load(context, R.raw.done03, 1));
		sounds.add(sound.load(context, R.raw.done04, 1));
		sounds.add(sound.load(context, R.raw.done05, 1));
		sounds.add(sound.load(context, R.raw.done06, 1));
*/
	}

	@Override
	public void run() {
		//player.start();

		while (!terminated) {
			try {
				if (!paused) update();
				draw();			
				Thread.sleep(1000/30);
			} catch (Exception e) {
				Log.e(e.getMessage(),  Log.getStackTraceString(e));
				break;
			}
		}
/*
		player.stop();
        player.release();
*/
        ((MainActivity)getContext()).finish();
	}
	
	public void terminate() {
		terminated = true;
	}
	
	public void setPause(boolean paused) {
		this.paused = paused;
/*
		try {
			if (paused) {
				if (player.isPlaying()) player.pause();
			} else {
				player.start();
			}

		} catch (IllegalStateException e) {
			Log.e(e.getMessage(),  Log.getStackTraceString(e));
		}
*/
	}
	
	private void update() {
		synchronized (hanabis) {
			for (Iterator<Hanabi> it = hanabis.iterator(); it.hasNext(); ) {
				Hanabi hanabi = it.next();
				if (hanabi.update()) {
					if (Screen.in(hanabi.getX(), hanabi.getY())) {
						for (int i = 0; i < Screen.BALL_MAX; i += 20) {
							FireBall ball = new FireBall();
							ball.setColor(255, 255, 0, 0);
							ball.setPosition(hanabi.getX(), hanabi.getY());
							ball.setRadius(10);
							ball.setVecter(10, i);
							ball.addVecter(hanabi.getDx(), hanabi.getDy());
							ball.setDecay(0.1f);
							balls.add(ball);
						}

						float scale = hanabi.getX() / Screen.WIDTH;
						sound.play(sounds.get(SOUND_ID_DONE),
								1 - scale, scale, 1, 0,
								Effect.getRate((int)(scale * 12)));
					}
					it.remove();

				} else {
					FireBall ball = new FireBall();
					ball.setColor(255, 240, 240, 200);
					ball.setPosition(hanabi.getX(), hanabi.getY());
					ball.setRadius(3);
					ball.addVecter(hanabi.getDx()/2, hanabi.getDy()/2);
					ball.setDecay(0.4f);
					balls.add(ball);
				}
			}
		}

		for (Iterator<FireBall> it = balls.iterator(); it.hasNext(); ) {
			FireBall ball = it.next();
			ball.update();
			if (ball.isLost()) it.remove();
		}
	}

	private void draw() {
		Canvas canvas = getHolder().lockCanvas();
		if (canvas != null) {
			try {
				canvas.drawColor(Color.WHITE);

				Screen.Info info = Screen.getInfo(canvas);
				canvas.translate(info.getOffsetX(), info.getOffsetY());
				canvas.scale(info.getScale(), info.getScale());

				canvas.clipRect(0, 0, Screen.WIDTH, Screen.HEIGHT);
				canvas.drawColor(Color.BLACK);
				
				synchronized (hanabis) {
					for (Hanabi hanabi: hanabis) {
						hanabi.draw(canvas);
					}
				}

				synchronized (balls) {
					for (FireBall ball: balls) {
						ball.draw(canvas);
					}
				}

			} finally {
				getHolder().unlockCanvasAndPost(canvas);
			}
		}
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		int action = MotionEventCompat.getActionMasked(event);
		int index = MotionEventCompat.getActionIndex(event);
		int id = MotionEventCompat.getPointerId(event, index);
		float x = MotionEventCompat.getX(event, index);
		float y = MotionEventCompat.getY(event, index);
		switch (action) {
		case MotionEvent.ACTION_DOWN:
		case MotionEvent.ACTION_POINTER_DOWN:
		{
			// タッチした点の処理
			Screen.Position pos = Screen.getScreenPosition(x, y);
			if (pos != null && Screen.in(pos.getX(), pos.getY())) {
				touchs.put(id, pos);
			}
			break;
		}
		case MotionEvent.ACTION_UP:
		case MotionEvent.ACTION_POINTER_UP:
			// 指を離した点の処理,
			synchronized (hanabis) {
				Screen.Position pos = Screen.getScreenPosition(x, y);
				Screen.Position trg = touchs.get(id);
				if (pos != null) {
					float dx = (pos.getX() - trg.getX()) / 2.0f;
					float dy = (pos.getY() - trg.getY()) / 2.0f;
					hanabis.add(new Hanabi(trg.getX(), trg.getY(), dx, dy));
					touchs.remove(id);
				}
			}
			break;
		case MotionEvent.ACTION_MOVE:
			// 指を動かした点の処理
			break;
		}
		//return super.onTouchEvent(event);
		return true;
	}
	
	@Override
	public void onWindowFocusChanged(boolean hasWindowFocus) {
		setPause(!hasWindowFocus);
		super.onWindowFocusChanged(hasWindowFocus);
	}
/*
	public void playBgm(boolean isPlay) {
		synchronized (player) {
			if (isPlay && !player.isPlaying())
				player.start();
			else if (player.isPlaying())
				player.pause();
		}
	}
*/
/*
	public void setBgm(File file) {
		synchronized (player) {
			//boolean playing = player.isPlaying();
			player.release();
			player = MediaPlayer.create(
					getContext(), Uri.fromFile(file));
			player.setLooping(true);
			player.start();
		}
	}
*/
}
